// fs_mapping.c

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/statvfs.h>
#include <mntent.h>
#include <rpm/rpmlog.h>
#include "fs_mapping.h"
#include "utils.h"

#define BUFFER_SIZE 1024

char* get_root_device() {
    FILE *fp;
    char buffer[BUFFER_SIZE];
    char *root_device = NULL;

    // 使用 popen 函数执行 mount 命令并读取其输出
    fp = popen("mount", "r");
    if (fp == NULL) {
        perror("popen");
        return NULL;
    }

    // 逐行读取 mount 命令的输出
    while (fgets(buffer, sizeof(buffer), fp) != NULL) {
        char device[BUFFER_SIZE];
        char mount_point[BUFFER_SIZE];
        char fs_type[BUFFER_SIZE];
        char options[BUFFER_SIZE];

        // 解析每一行的输出
        if (sscanf(buffer, "%s on %s type %s (%[^)])", device, mount_point, fs_type, options) == 4) {
            // 检查挂载点是否为根目录
            if (strcmp(mount_point, "/") == 0) {
                root_device = strdup(device);
                break;
            }
        }
    }

    // 关闭 popen 流
    pclose(fp);

    return root_device;
}

int remount_root_rw(const char *user_dir, char **out_temp_dir) {
    struct statvfs vfs_stat;
    if (statvfs("/", &vfs_stat) != 0) {
        rpmlog(RPMLOG_ERR, "Failed to get filesystem stats for /.\n");
        return 1;
    }

    // Check if the root directory is read-only
    if (!(vfs_stat.f_flag & ST_RDONLY)) {
        rpmlog(RPMLOG_DEBUG ,"Root directory is not read-only\n");
        return 0;
    }

    struct stat root_stat;
    if (stat("/", &root_stat) != 0) {
        rpmlog(RPMLOG_ERR, "Cannot get the / stat\n");
        return 1;
    }

    FILE *mounts = setmntent("/proc/mounts", "r");
    if (mounts == NULL) {
        rpmlog(RPMLOG_ERR, "Cannot open /proc/mounts\n");
        return 1;
    }

    struct mntent *ent;
    char *device = NULL;
    while ((ent = getmntent(mounts)) != NULL) {
        struct stat ent_stat;
        if (stat(ent->mnt_fsname, &ent_stat) != 0) {
            continue;
        }

        if (ent_stat.st_rdev == root_stat.st_dev) {
            device = ent->mnt_fsname;
            break;
        }
    }

    endmntent(mounts);

    if (device == NULL) {
        device = get_root_device();
        if (device == NULL) {
            fprintf(stderr, "Could not find device for root directory\n");
            return 1;
        }
    }

    char tmp_root_dir[PATH_MAX];
    snprintf(tmp_root_dir, sizeof(tmp_root_dir), "%s/tmp_root_XXXXXX", user_dir);
    char *temp_dir = mkdtemp(tmp_root_dir);

    if (temp_dir == NULL) {
        rpmlog(RPMLOG_ERR, "Cannot create temporary root directory\n");
        return 1;
    }

    char command[256];
    snprintf(command, sizeof(command), "mount -o rw %s %s", device, temp_dir);
    if (system(command) != 0) {
        rpmlog(RPMLOG_ERR, "Cannot mount temporary root directory\n");
        return 1;
    }

    rpmlog(RPMLOG_DEBUG, "Remounting the rootfs to %s ..\n", temp_dir);

    char command2[256];
    snprintf(command2, sizeof(command2), "mount -o remount,rw %s", temp_dir);
    if (system(command2) != 0) {
        rpmlog(RPMLOG_ERR, "Cannot remount temporary root directory\n");
        return 1;
    }

    *out_temp_dir = strdup(temp_dir);

    return 0;
}