// main.c

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <rpm/rpmlog.h>
#include <getopt.h>
#include "main.h"
#include "config.h"
#include "rpm_operations.h"
#include "fs_mapping.h"
#include "sandbox.h"
#include "dnf_py/dnf_wrapper.h"

void print_usage(const char *prog_name);

int main(int argc, char *argv[]) {
    const char *root_path = NULL;
    // 配置文件路径, TODO
    const char *config_file = "/etc/rpm-silo.conf";
    Config config;
    int debug_mode = 0; // 标记是否启用调试模式
    int use_dnf = 0;    // 标记是否使用dnf

    // 初始化配置
    if (read_config(config_file, &config) != 0) {
        rpmlog(RPMLOG_WARNING, "无法读取配置文件 %s，使用默认配置。\n", config_file);
        init_default_config(&config);
    }

    // 默认根路径
    root_path = config.root_path;

    // 定义选项
    int opt;
    int option_index = 0;
    static struct option long_options[] = {
        {"root",  required_argument, 0,  'r' },
        {"debug", no_argument,       0,  'd' },
        {"dnf",   no_argument,       0,  'D' },
        {"help",  no_argument,       0,  'h' },
        {0,       0,                 0,   0  }
    };

    // 解析命令行参数
    while ((opt = getopt_long(argc, argv, "r:dDh", long_options, &option_index)) != -1) {
        switch (opt) {
            case 'r':
                root_path = optarg;
                break;
            case 'd':
                debug_mode = 1;
                break;
            case 'D':
                use_dnf = 1;
                break;
            case 'h':
                print_usage(argv[0]);
                return 0;
            case '?':
                // getopt_long 已经打印了错误信息
                return 1;
            default:
                print_usage(argv[0]);
                return 1;
        }
    }

    // 如果启用调试模式，设置日志级别
    if (debug_mode) {
        rpmlogSetMask(RPMLOG_UPTO(RPMLOG_DEBUG));
        rpmlog(RPMLOG_DEBUG, "已启用调试模式。\n");
    } else {
        rpmlogSetMask(RPMLOG_UPTO(RPMLOG_INFO));
    }

    // 检查是否指定了操作
    if (optind >= argc) {
        rpmlog(RPMLOG_ERR, "未指定操作。\n");
        print_usage(argv[0]);
        return 1;
    }

    // 解析操作和参数
    const char *operation = argv[optind];
    optind++;

    // 复制 RPM 数据库
    if (copy_rpmdb(root_path) != 0) {
        rpmlog(RPMLOG_ERR, "复制 RPM 数据库失败。\n");
        return 1;
    }

    if (strcmp(operation, "install") == 0) {
        if (optind >= argc) {
            rpmlog(RPMLOG_ERR, "请指定要安装的包或 RPM 文件。\n");
            return 1;
        }

        int pkg_count = argc - optind;
        const char **pkg_list = (const char **)malloc(pkg_count * sizeof(char *));
        if (pkg_list == NULL) {
            rpmlog(RPMLOG_ERR, "内存分配失败。\n");
            return 1;
        }

        // 收集包名或 RPM 文件路径
        for (int i = 0; i < pkg_count; i++) {
            pkg_list[i] = argv[optind + i];
        }

        if (debug_mode) {
            rpmlog(RPMLOG_DEBUG, "待安装的包列表：\n");
            for (int i = 0; i < pkg_count; i++) {
                rpmlog(RPMLOG_DEBUG, "  %s\n", pkg_list[i]);
            }
        }

        // 调用相应的安装函数
        int result = 0;
        if (use_dnf) {
            // use dnf download the rpms
            char rpmdb_path[PATH_MAX];
            int result_count;
            snprintf(rpmdb_path, PATH_MAX, "%s%s", root_path, SYSTEM_RPMDB);

            char **installed_packages = call_install_package(pkg_list, pkg_count, root_path, rpmdb_path, &result_count);

            if (installed_packages != NULL && result_count > 0) {
                rpmlog(RPMLOG_DEBUG, "Installing %d packages using repo...\n", result_count);
                install_packages(installed_packages, result_count, root_path);

                // 释放内存
                for (int i = 0; i < result_count; ++i) {
                    free(installed_packages[i]);
                }
                free(installed_packages);
            } else {
                rpmlog(RPMLOG_ERR, "Failed to install packages\n");
                return 1;
            }

            //result = dnf_install_packages(pkg_list, pkg_count, root_path);
        } else {
            result = install_packages(pkg_list, pkg_count, root_path);
        }

        // 释放内存
        free(pkg_list);

        if (result != 0) {
            rpmlog(RPMLOG_ERR, "安装包失败。\n");
            return 1;
        }

    } else if (strcmp(operation, "uninstall") == 0 || strcmp(operation, "remove") == 0) {
        if (optind >= argc) {
            rpmlog(RPMLOG_ERR, "请指定要卸载的包名称。\n");
            return 1;
        }
        const char **package_names = (const char **) &argv[optind];
        int pkg_count = argc - optind;

        if (debug_mode) {
            for (int i = 0; i < pkg_count; i++) {
                rpmlog(RPMLOG_DEBUG, "待卸载的包：%s\n", package_names[i]);
            }
        }

        // 调用相应的卸载函数
        int result = 0;
        if (use_dnf) {
            //result = dnf_uninstall_packages(package_names,  pkg_count, root_path);
            // we use this for now.
            result = uninstall_packages(package_names,  pkg_count, root_path);
        } else {
            result = uninstall_packages(package_names,  pkg_count, root_path);
        }

        if (result != 0) {
            rpmlog(RPMLOG_ERR, "卸载包失败。\n");
            return 1;
        }

    } else if (strcmp(operation, "query") == 0) {
        if (optind >= argc) {
            rpmlog(RPMLOG_ERR, "请指定要查询的包名称。\n");
            return 1;
        }
        const char **package_names = (const char **) &argv[optind];
        int pkg_count = argc - optind;

        if (debug_mode) {
            for (int i = 0; i < pkg_count; i++) {
                rpmlog(RPMLOG_DEBUG, "待查询的包：%s\n", package_names[i]);
            }
        }

        // 调用相应的查询函数
        int result = 0;
        if (use_dnf) {
            //result = dnf_query_packages(package_names, pkg_count, root_path);
            call_query_package(package_names, pkg_count);
        } else {
            result = query_packages(package_names, pkg_count, root_path);
        }

        if (result != 0) {
            rpmlog(RPMLOG_ERR, "查询包信息失败。\n");
            return 1;
        }

    } else {
        rpmlog(RPMLOG_ERR, "未知操作：%s\n", operation);
        print_usage(argv[0]);
        return 1;
    }

    return 0;
}

void print_usage(const char *prog_name) {
    printf("用法：%s [OPTIONS] <操作> [参数]\n", prog_name);
    printf("选项：\n");
    printf("  --root <路径>, -r <路径>    指定根路径\n");
    printf("  --debug, -d                 启用调试模式\n");
    printf("  --dnf, -D                   从远程仓库操作（使用dnf）\n");
    printf("  --help, -h                  显示此帮助信息\n");
    printf("操作：\n");
    printf("  install <包名或rpm文件>...  安装一个或多个包\n");
    printf("  uninstall <包名>            卸载包\n");
    printf("  remove <包名>               卸载包\n");
    printf("  query <包名>                查询包信息\n");
}
