// rpm_operations.c

#include <stdio.h>
#include <rpm/rpmlib.h>
#include <rpm/rpmio.h>
#include <rpm/rpmps.h>
#include <rpm/rpmfi.h>
#include <rpm/rpmlog.h>
#include <rpm/rpmdb.h>
#include <rpm/header.h>
#include <rpm/rpmmacro.h>
#include <rpm/rpmtypes.h>
#include <rpm/rpmprob.h>
#include <rpm/rpmcli.h>
#include <glob.h>
#include <string.h>
#include <limits.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdbool.h>
#include <errno.h>
#include "rpm_operations.h"
#include "utils.h"
#include "fs_mapping.h"

#define MAX_ERROR_MSG_LEN 4096
#define RPMDB_LOCATION "/usr/lib/sysimage/rpm/"

static char error_messages[MAX_ERROR_MSG_LEN];

#define RPMSCRIPT_PRETRANS 1
#define RPMSCRIPT_PREIN    2
#define RPMSCRIPT_POSTIN   3
#define RPMSCRIPT_PREUN    4
#define RPMSCRIPT_POSTUN   5
#define RPMSCRIPT_POSTTRANS 6

typedef struct {
    FD_t current_fd;
    const char *tmpdir_path;
} TransactionData;

typedef struct HeaderNode {
    Header header;
    struct HeaderNode *next;
} HeaderNode;

void * my_rpm_callback(const void *h, rpmCallbackType what,
                       rpm_loff_t amount, rpm_loff_t total,
                       fnpyKey key, rpmCallbackData data)
{
    TransactionData *tdata = (TransactionData *)data;

    switch (what) {
        case RPMCALLBACK_INST_OPEN_FILE: {
            // 假设 key 是包文件的相对路径
            const char *pkg_file_path = (const char *)key;

            // 构建完整的文件路径
            // char full_path[PATH_MAX];
            // snprintf(full_path, sizeof(full_path), "%s/%s", tdata->tmpdir_path, pkg_file_path);

            // 确保没有已打开的文件描述符
            if (tdata->current_fd != NULL) {
                Fclose(tdata->current_fd);
                tdata->current_fd = NULL;
            }

            // 打开文件
            tdata->current_fd = Fopen(pkg_file_path, "r.ufdio");
            if (tdata->current_fd == NULL) {
                rpmlog(RPMLOG_ERR, "无法打开文件：%s\n", pkg_file_path);
                return NULL;
            }

            // fprintf(stderr,"打开文件：%s\n", pkg_file_path);

            return tdata->current_fd;
        }

        case RPMCALLBACK_INST_CLOSE_FILE: {
            if (tdata->current_fd != NULL) {
                Fclose(tdata->current_fd);
                tdata->current_fd = NULL;
            }
            break;
        }

        // 可以根据需要添加其他回调类型的处理

        default:
            break;
    }

    return NULL;
}

int copy_rpmdb(const char *root_path) {
    char source_db[PATH_MAX];
    char dest_db[PATH_MAX];

    snprintf(source_db, PATH_MAX, "%s", SYSTEM_RPMDB);
    snprintf(dest_db, PATH_MAX, "%s%s", root_path, SYSTEM_RPMDB);

    // 检查目标数据库是否已存在
    if (access(dest_db, F_OK) == 0) {
        // 数据库已存在，跳过复制
        rpmlog(RPMLOG_DEBUG, "Target RPMDB directory already exists, skip copying: %s\n", dest_db);
        return 0;
    }

    rpmlog(RPMLOG_DEBUG, "Copying RPMDB directory from %s to %s\n", source_db, dest_db);

    // 创建目标目录
    if (create_parent_dirs(dest_db) != 0) {
        rpmlog(RPMLOG_ERR, "Can't create RPMDB directory %s\n", dest_db);
        return -1;
    }

    // 复制数据库目录
    if (copy_file(source_db, dest_db) != 0) {
        rpmlog(RPMLOG_ERR, "Failed to copy RPMDB.\n");
        return -1;
    }

    return 0;
}

// 定义 set_rpm_macro_define 函数
static void set_rpm_macro_define(const char *key, const char *value) {
    // 计算所需的缓冲区大小
    size_t key_len = strlen(key);
    size_t value_len = strlen(value);
    size_t buf_size = strlen("%define ") + key_len + value_len + 2; // +2 for space and null terminator

    // 分配内存
    char *buf = (char *)malloc(buf_size);
    if (buf == NULL) {
        fprintf(stderr, "Memory allocation failed\n");
        return;
    }

    // 构建宏定义字符串
    sprintf(buf, "%s%s %s", "%define ", key, value);

    // 调用 rpmExpand 函数来处理宏定义字符串
    // 忽略返回值
    free(rpmExpand(buf, NULL));

    // 释放分配的内存
    free(buf);
}


// copy from psm.c
static int isUpdate(rpmts ts, rpmte te)
{
    rpmtsi pi = rpmtsiInit(ts);
    rpmte p;
    int update = 0;
    while ((p = rpmtsiNext(pi, TR_REMOVED)) != NULL) {
	if (rpmteDependsOn(p) == te) {
	    update = 1;
	    break;
	}
    }
    rpmtsiFree(pi);
    return update;
}

static int octryReadHeader(rpmts ts, char * rpm_file, Header * header) {
    rpmRC rc;

    FD_t fd = Fopen(rpm_file, "r.ufdio");
    if (fd == NULL) {
        rpmlog(RPMLOG_ERR, "无法打开文件 %s\n", rpm_file);
        return RPMRC_FAIL;
    }

    rc = rpmReadPackageFile(ts, fd, rpm_file, header);
    Fclose(fd);

    if (rc != RPMRC_OK || *header == NULL) {
        fprintf(stderr, "无法读取 RPM 包头信息：%s\n", rpm_file);
        return RPMRC_FAIL;
    }

    // 获取包名并打印
    const char *pkgName = headerGetString(*header, RPMTAG_NAME);
    if (pkgName != NULL) {
        rpmlog(RPMLOG_DEBUG, "Package Name: %s\n", pkgName);
    } else {
        rpmlog(RPMLOG_ERR, "Failed to get package name.\n");
    }

    return RPMRC_OK;
}

int install_files_to_rootpath(rpmts ts, const char *root_path) {
    rpmRC rc;

    // 设置文件安装的根路径
    rc = rpmtsSetRootDir(ts, root_path);
    if (rc != RPMRC_OK) {
        rpmlog(RPMLOG_ERR, "无法设置文件安装的根路径\n");
        return -1;
    }

    // 设置事务标志，禁用自动执行脚本
    rpmtsSetFlags(ts, RPMTRANS_FLAG_NOSCRIPTS | RPMTRANS_FLAG_NOCONTEXTS);

    // 执行事务，将文件安装到 root_path 下
    rc = rpmtsRun(ts, NULL, 0);
    if (rc != RPMRC_OK) {
        rpmlog(RPMLOG_ERR, "安装失败\n");
        return -1;
    } else {
        rpmlog(RPMLOG_INFO, "文件已安装到 %s\n", root_path);
        return 0;
    }
}

int execute_script(Header h, int script_type, const char *script_name, int arg1, int arg2) {
    int result = 0;

    // 确定脚本内容和解释器对应的标签
    int script_tag = RPMTAG_NOT_FOUND;
    int prog_tag = RPMTAG_NOT_FOUND;

    switch (script_type) {
        case RPMSCRIPT_PREIN:
            script_tag = RPMTAG_PREIN;
            prog_tag = RPMTAG_PREINPROG;
            break;
        case RPMSCRIPT_POSTIN:
            script_tag = RPMTAG_POSTIN;
            prog_tag = RPMTAG_POSTINPROG;
            break;
        case RPMSCRIPT_PREUN:
            script_tag = RPMTAG_PREUN;
            prog_tag = RPMTAG_PREUNPROG;
            break;
        case RPMSCRIPT_POSTUN:
            script_tag = RPMTAG_POSTUN;
            prog_tag = RPMTAG_POSTUNPROG;
            break;
        case RPMSCRIPT_POSTTRANS:
            script_tag = RPMTAG_POSTTRANS;
            prog_tag = RPMTAG_POSTTRANSPROG;
            break;
        default:
            rpmlog(RPMLOG_ERR, "不支持的脚本类型\n");
            return -1;
    }

    // 获取脚本内容
    rpmtd script_td = rpmtdNew();
    if (!headerGet(h, script_tag, script_td, HEADERGET_DEFAULT)) {
        // 没有对应的脚本，跳过
        rpmlog(RPMLOG_DEBUG, "无 %d 脚本\n", script_tag);
        rpmtdFree(script_td);
        return 0;
    }

    const char *script = rpmtdGetString(script_td);
    rpmlog(RPMLOG_DEBUG, "脚本内容: %s\n", script);
    if (script == NULL || strlen(script) == 0) {
        // 脚本内容为空，跳过
        rpmtdFree(script_td);
        return 0;
    }

    // 获取脚本解释器
    const char *interpreter = headerGetString(h, prog_tag);
    if (interpreter == NULL || strlen(interpreter) == 0) {
        // 默认使用 "/bin/sh"
        interpreter = "/bin/bash";
    }

    // 创建临时脚本文件
    char script_path[] = "/tmp/rpm_scriptXXXXXX";
    int fd = mkstemp(script_path);
    if (fd < 0) {
        perror("无法创建临时脚本文件");
        rpmtdFree(script_td);
        return -1;
    }

    // 写入脚本内容
    ssize_t write_size = write(fd, script, strlen(script));
    if (write_size < 0) {
        perror("写入脚本内容失败");
        close(fd);
        unlink(script_path);
        rpmtdFree(script_td);
        return -1;
    }

    close(fd);
    rpmtdFree(script_td);

    // 设置脚本文件的可执行权限
    if (chmod(script_path, 0755) != 0) {
        perror("设置脚本文件权限失败");
        unlink(script_path);
        return -1;
    }

    // 构建命令行参数
    ARGV_t argv = NULL;
    argvAdd(&argv, interpreter);
    argvAdd(&argv, script_path);

    if (arg1 >= 0) {
	    argvAddNum(&argv, arg1);
	}
	if (arg2 >= 0) {
	    argvAddNum(&argv, arg2);
	}

    rpmlog(RPMLOG_DEBUG, "正在执行脚本: %s %s %s\n", argv[0], argv[1], argv[2]);

    // 在根目录下执行脚本
    pid_t pid = fork();
    if (pid == 0) {
        // 子进程
        // 切换到根目录
        if (chdir("/") != 0) {
            perror("无法切换到根目录");
            exit(EXIT_FAILURE);
        }
        // 执行脚本
        execv(interpreter, (char * const *)argv);
        // 如果 execv 返回，表示出错
        perror("execv 失败");
        exit(EXIT_FAILURE);
    } else if (pid > 0) {
        // 父进程，等待子进程完成
        int status = 0;
        if (waitpid(pid, &status, 0) == -1) {
            perror("等待子进程失败");
            result = -1;
        } else {
            if (WIFEXITED(status)) {
                int exit_status = WEXITSTATUS(status);
                if (exit_status != 0) {
                    rpmlog(RPMLOG_ERR, "%s 执行失败，退出状态：%d\n", script_name, exit_status);
                    result = -1;
                }
            } else {
                rpmlog(RPMLOG_ERR, "%s 执行异常\n", script_name);
                result = -1;
            }
        }
    } else {
        // fork 失败
        perror("fork 失败");
        result = -1;
    }

    // 删除临时脚本文件
    unlink(script_path);

    return result;
}

int execute_scripts_in_root(rpmts ts, char ** rpm_files, Header header, int script_type) {
    int n_rpmts_elements = rpmtsNElements(ts);
    rpmRC rc = RPMRC_OK;
    rpmlog(RPMLOG_DEBUG, "n_rpmts_elements: %d\n", n_rpmts_elements);

    // 执行脚本
    for (int i = 0; i < n_rpmts_elements; i++) {
        rpmte te = rpmtsElement(ts, i);
        Header h = NULL;

        if (rpm_files != NULL) {
            const char *rpm_file = rpm_files[i];
            octryReadHeader(ts, rpm_file, &h);
        } else {
            h = header;
        }

        const char *pkg_name = headerGetString(h, RPMTAG_NAME);
        if (pkg_name == NULL) {
            pkg_name = "(unknown)";
        }

        int arg1 = rpmdbCountPackages(rpmtsGetRdb(ts), pkg_name);
        // TODO: understood what can countCorrection do
        int countCorrection = 0;

        rpmlog(RPMLOG_DEBUG, "Pkg count: %d\n", arg1);

        switch (rpmteType(te))
        {
        case PKG_INSTALL:
        case PKG_PRETRANS:
            arg1 += 1;
            break;
        case PKG_ERASE:
            arg1 -= 1;
            break;
        case PKG_VERIFY:
        case PKG_POSTTRANS:
            arg1 += isUpdate(ts, te);
            break;
        default:
            break;
        }

        const char *script_name = "";
        switch (script_type) {
            case RPMSCRIPT_PREIN:
                script_name = "预安装脚本";
                break;
            case RPMSCRIPT_POSTIN:
                script_name = "后安装脚本";
                break;
            case RPMSCRIPT_POSTTRANS:
                script_name = "后转换脚本";
                // 如果是卸载后的转换脚本，则arg1要加1
                if (arg1 < 0) {
                    countCorrection = 1;
                }
                break;
            case RPMSCRIPT_PREUN:
                script_name = "预卸载脚本";
                break;
            case RPMSCRIPT_POSTUN:
                script_name = "后卸载脚本";
                countCorrection = 1;
                break;
            default:
                script_name = "未知脚本";
                break;
        }

        arg1 += countCorrection;

        rpmlog(RPMLOG_DEBUG, "执行%s：%s\n", script_name, pkg_name);
        // https://docs.fedoraproject.org/en-US/packaging-guidelines/Scriptlets/
        // This argument, accessed via $1 (for shell scripts) is the number of
        // packages of this name which will be left on the system when the
        // action completes
        int arg2 = -1;
        if (arg1 < 0) {
            /* XXX W2DO? fails as "execution of script failed" */
            rpmlog(RPMLOG_ERR, "包计数错误：%s, %d\n", pkg_name, arg1);
            rc = RPMRC_FAIL;
	    } else {
            if (execute_script(h, script_type, script_name, arg1, arg2) != 0) {
                rpmlog(RPMLOG_ERR, "无法执行%s：%s\n", script_name, pkg_name);
                rc = RPMRC_FAIL;
            }
        }
    }

    return rc;
}

int create_hard_links_to_root(rpmts ts, char ** rpm_files, const char *tmp_root, const char *root_path) {

    int n_rpmts_elements = rpmtsNElements(ts);

    // 确保源文件存在
    if (access(tmp_root, F_OK) != 0) {
        rpmlog(RPMLOG_ERR, "临时根目录不存在：%s\n", tmp_root);
        return -1;
    }

    // 遍历事务元素，获取已安装的文件列表
    for (int i = 0; i < n_rpmts_elements; i++) {
        const char *rpm_file = rpm_files[i];
        rpmlog(RPMLOG_DEBUG, "创建硬链接: %s\n", rpm_file);
        rpmte te = rpmtsElement(ts, i);
        if (rpmteType(te) != TR_ADDED) {
            continue;
        }

        Header header = NULL;
        octryReadHeader(ts, rpm_file, &header);
        rpmfi fi = rpmfiNew(NULL, header, RPMTAG_BASENAMES, RPMFI_NOHEADER | RPMFI_FLAGS_QUERY);
        if (fi == NULL) {
            rpmlog(RPMLOG_DEBUG, "无法创建 rpmfi 对象\n");
            continue;
        }

        // 遍历文件列表
        while (rpmfiNext(fi) >= 0) {
            const char *filepath = rpmfiFN(fi);
            rpmlog(RPMLOG_DEBUG, "文件路径：%s\n", filepath);

            // 排除目录
            mode_t mode = rpmfiFMode(fi);
            if (S_ISDIR(mode)) {
                continue;
            }

            // 构建源文件路径和目标文件路径
            char source_path[PATH_MAX];
            char dest_path[PATH_MAX];

            snprintf(source_path, PATH_MAX, "%s/%s", root_path, filepath);
            snprintf(dest_path, PATH_MAX, "%s/%s", tmp_root, filepath);

            // 确保源文件存在
            if (access(source_path, F_OK) != 0) {
                rpmlog(RPMLOG_ERR, "源文件不存在：%s\n", source_path);
                continue;
            }

            // 确保目标目录存在
            if (create_parent_dirs(dest_path) != 0) {
                rpmlog(RPMLOG_ERR, "无法创建目录：%s\n", dest_path);
                continue;
            }

            // 删除目标文件（如果已存在）
            unlink(dest_path);

            // 创建硬链接
            // 在remount之后不能创建硬链接，所以这里我们改为软连接
            if (symlink(source_path, dest_path) != 0) {
                rpmlog(RPMLOG_ERR, "创建硬链接失败, 源文件：%s，目标文件：%s\n", source_path, dest_path);
                continue;
            }
            rpmlog(RPMLOG_DEBUG, "源文件：%s，目标文件：%s\n", source_path, dest_path);
        }

        rpmfiFree(fi);
    }

    return 0;
}

char ** record_hardlinks_to_remove(rpmts ts, char ** rpm_files, int * file_count) {
    char * const * arg;
    char **file_list = NULL;
    int count = 0;
    int capacity = 10; // 初始容量

    file_list = (char **)malloc(capacity * sizeof(char *));
    if (file_list == NULL) {
        rpmlog(RPMLOG_ERR, "内存分配失败\n");
        return NULL;
    }


    for (arg = rpm_files; *arg; arg++) {
        const char *pkg_name = *arg;
        rpmdbMatchIterator mi = rpmtsInitIterator(ts, RPMDBI_LABEL, *arg, 0);
        int matches = rpmdbGetIteratorCount(mi);
        int erasing = 1;
        int result = 0;

        if (matches == 0) {
            rpmlog(RPMLOG_ERR, "Something wrong happened when removing %s. \n", pkg_name);
        } else {
            Header h; // 迭代器拥有引用

            if (matches > 1) {
                rpmlog(RPMLOG_ERR, "\"%s\" 匹配了多个包：\n", pkg_name);
            }

            while ((h = rpmdbNextIterator(mi)) != NULL) {
                char *qfmt = NULL;
                qfmt = rpmExpand("%{?_query_all_fmt}\n", NULL);
                char *nevra = headerFormat(h, qfmt, NULL);
                rpmlog(RPMLOG_INFO, "  %s", nevra);
                free(nevra);

                // 依赖检查
                rpmps ps;

                if (rpmtsCheck(ts)) {
                    result = -1;
                }

                ps = rpmtsProblems(ts);
                if (result != -1 && rpmpsNumProblems(ps) > 0) {
                    rpmlog(RPMLOG_ERR, "卸载依赖问题:\n");
                    rpmpsPrint(NULL, ps);
                    result = -1;
                }
                rpmpsFree(ps);

                rpmfi fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, RPMFI_NOHEADER | RPMFI_FLAGS_QUERY);

                if (fi == NULL) {
                    rpmlog(RPMLOG_DEBUG, "无法创建 rpmfi 对象\n");
                    continue;
                }

                // 遍历文件列表
                while (rpmfiNext(fi) >= 0) {
                    const char *filepath = rpmfiFN(fi);
                    rpmlog(RPMLOG_DEBUG, "文件路径：%s\n", filepath);

                    // 排除目录
                    mode_t mode = rpmfiFMode(fi);
                    if (S_ISDIR(mode)) {
                        continue;
                    }

                    // 构建源文件路径和目标文件路径
                    char dest_path[PATH_MAX];
                    snprintf(dest_path, PATH_MAX, "/%s", filepath);

                    // 将文件路径添加到列表中
                    if (count >= capacity) {
                        capacity *= 2;
                        file_list = (char **)realloc(file_list, capacity * sizeof(char *));
                        if (file_list == NULL) {
                            rpmlog(RPMLOG_ERR, "内存重新分配失败\n");
                            rpmfiFree(fi);
                            rpmdbFreeIterator(mi);
                            return NULL;
                        }
                    }

                    file_list[count] = strdup(dest_path);
                    if (file_list[count] == NULL) {
                        rpmlog(RPMLOG_ERR, "内存分配失败\n");
                        rpmfiFree(fi);
                        rpmdbFreeIterator(mi);
                        return NULL;
                    }
                    count++;

                    rpmlog(RPMLOG_DEBUG, "记录硬链接：%s\n", dest_path);
                }

                rpmfiFree(fi);

            }
        }
        rpmdbFreeIterator(mi);
    }

    *file_count = count; // 传递文件数量
    return file_list;
}

int remove_hard_links_from_root(char ** rpm_files, int file_count, const char *tmp_root_dir) {
    rpmlog(RPMLOG_DEBUG, "删除硬链接\n");

    // 遍历事务元素，获取要卸载的文件列表
    for (int i = 0; i < file_count; i++) {
        // 遍历文件列表
        rpmlog(RPMLOG_DEBUG, "文件路径：%s%s\n", tmp_root_dir, rpm_files[i]);


        // 删除硬链接
        char remove_path[PATH_MAX];
        snprintf(remove_path, sizeof(remove_path), "%s%s", tmp_root_dir, rpm_files[i]);

        if (unlink(remove_path) != 0) {
            rpmlog(RPMLOG_ERR, "无法删除硬链接：%s\n", remove_path);
            continue;
        }

        rpmlog(RPMLOG_DEBUG, "已删除硬链接：%s\n", remove_path);

        free(rpm_files[i]);
    }

    free(rpm_files);

    return 0;
}

int install_packages(const char **rpm_files, int rpm_file_count, const char *root_path) {
    rpmts ts = NULL;
    rpmRC rc;
    int result = 0;

    // 初始化 RPM 库
    rpmReadConfigFiles(NULL, NULL);

    // 创建事务集
    ts = rpmtsCreate();
    if (ts == NULL) {
        rpmlog(RPMLOG_ERR, "无法创建事务集\n");
        return -1;
    }

    // 设置根路径
    rc = rpmtsSetRootDir(ts, root_path);
    if (rc != 0) {
        rpmlog(RPMLOG_ERR, "无法设置根路径\n");
        result = -1;
        goto cleanup;
    }

    rpmtsSetVSFlags (ts, _RPMVSF_NOSIGNATURES | _RPMVSF_NODIGESTS);

    // 设置验证标志
    rpmtsSetVfyLevel (ts, 0);

    // 设置回调函数和数据
    TransactionData tdata;
    tdata.current_fd = NULL;
    tdata.tmpdir_path = NULL; // 根据实际情况设置
    rpmtsSetNotifyCallback(ts, my_rpm_callback, &tdata);
    // rpmtsSetNotifyCallback (ts, rpmShowProgress, INSTALL_INSTALL);

    // 添加安装元素
    for (int i = 0; i < rpm_file_count; i++) {
        const char *rpm_file = rpm_files[i];
        Header header = NULL;

        octryReadHeader(ts, rpm_file, &header);

        if (header == NULL) {
            rpmlog(RPMLOG_ERR, "Header 为空，无法添加安装元素：%s\n", rpm_file);
            continue;
        }

        // 添加安装元素
        rpmlog(RPMLOG_DEBUG, "Adding install element: %s\n", (fnpyKey)rpm_file);
        rc = rpmtsAddInstallElement(ts, header, rpm_file, 0, NULL);
        //headerFree(header);

        if (rc != RPMRC_OK) {
            rpmlog(RPMLOG_ERR, "无法添加安装元素：%s\n", rpm_file);
            continue;
        }

    }

    // 依赖检查
    rpmps ps;

    if (rpmtsCheck(ts)) {
        result = -1;
    }

    ps = rpmtsProblems(ts);
    if (result != -1 && rpmpsNumProblems(ps) > 0) {
        rpmlog(RPMLOG_ERR, "Failed dependencies:\n");
        rpmpsPrint(NULL, ps);
        result = -1;
    }
    rpmpsFree(ps);

    if (rpmtsOrder(ts)) {
        result = -1;
    }

    // TODO：添加事务后统一提交执行，拆分开
    // 执行安装事务
    if (result != -1) {

        // 执行 `%pre` 脚本
        if (execute_scripts_in_root(ts, rpm_files, NULL, RPMSCRIPT_PREIN) != RPMRC_OK) {
            result = -1;
            goto cleanup;
        }

        // 执行安装事务
        rpmFlags flags = 0;
        flags |= RPMPROB_FILTER_DISKSPACE;
        flags |= RPMPROB_FILTER_OLDPACKAGE;

        rpmtsSetFlags (ts, RPMTRANS_FLAG_NOSCRIPTS);
        rpmtsClean(ts);
        rc = rpmtsRun(ts, NULL, flags);
        if (rc != RPMRC_OK) {
            rpmlog(RPMLOG_ERR, "安装失败：\n");

            // 获取事务的问题集合
            rpmps problems = rpmtsProblems(ts);

            if (problems != NULL) {
                if (rpmpsNumProblems(problems) > 0) {
                    // rpmlog(RPMLOG_ERR, "Failed dependencies:\n");
                    rpmpsPrint(NULL, problems);
                }
                rpmpsFree(problems);
            } else {
                rpmlog(RPMLOG_ERR, "未能获取详细的错误信息。\n");
            }

            rpmtsFree(ts);
            return -1;
        } else {
            rpmlog(RPMLOG_INFO, "安装成功\n");
        }

        // remount the root, then create the hardlinks
        char *temp_dir = NULL;
        if (remount_root_rw(root_path, &temp_dir) != 0) {
            rpmlog(RPMLOG_ERR, "Failed to remount the root。\n");
            result = -1;
            goto cleanup;
        }

        // if temp_dir is NULL, we use the root directly
        if (temp_dir == NULL) {
            temp_dir = "/";
        }

        // 创建硬链接到根目录
        if (create_hard_links_to_root(ts, rpm_files, temp_dir, root_path) != 0) {
            result = -1;
            goto cleanup;
        }

        // 执行 `%post` 脚本
        if (execute_scripts_in_root(ts, rpm_files, NULL, RPMSCRIPT_POSTIN) != 0) {
            result = -1;
            goto cleanup;
        }

        // 执行 `%posttrans` 脚本
        if (execute_scripts_in_root(ts, rpm_files, NULL, RPMSCRIPT_POSTTRANS) != 0) {
            result = -1;
            goto cleanup;
        }

        // umount the root, then remove the temporary directory
        if (temp_dir != NULL && temp_dir != "/") {
            if(umount_and_remove(temp_dir)){
                result = -1;
                goto cleanup;
            }
        }
    }

cleanup:
    if (ts != NULL) {
        rpmtsFree(ts);
    }
    return result;
}

int uninstall_packages(const char **pkg_names, int pkg_count, const char *root_path) {
    rpmts ts = NULL;
    rpmRC rc;
    int result = 0;
    char * const * arg;
    int numFailed = 0;
    int numPackages = 0;
    char *qfmt = NULL;
    char ** filelist = NULL;

    // 初始化 RPM 库
    rpmReadConfigFiles(NULL, NULL);

    // 创建事务集
    ts = rpmtsCreate();
    if (ts == NULL) {
        rpmlog(RPMLOG_ERR, "无法创建事务集\n");
        return -1;
    }

    // 设置根路径
    rc = rpmtsSetRootDir(ts, root_path);
    if (rc != 0) {
        rpmlog(RPMLOG_ERR, "无法设置根路径\n");
        result = -1;
        goto cleanup;
    }

    //rpmtsSetVSFlags(ts, _RPMVSF_NOSIGNATURES | _RPMVSF_NODIGESTS);

    // 设置验证标志
    rpmtsSetVfyLevel(ts, 0);

    // 设置回调函数和数据
    TransactionData tdata;
    tdata.current_fd = NULL;
    tdata.tmpdir_path = NULL; // 根据实际情况设置
    rpmtsSetNotifyCallback(ts, my_rpm_callback, &tdata);

    // 获取查询格式，用于错误信息输出
    qfmt = rpmExpand("%{?_query_all_fmt}\n", NULL);

    HeaderNode *head = NULL; // 链表头
    HeaderNode *tail = NULL; // 链表尾

    for (arg = pkg_names; *arg; arg++) {
        const char *pkg_name = *arg;
        rpmdbMatchIterator mi = rpmtsInitIterator(ts, RPMDBI_LABEL, *arg, 0);
        int matches = rpmdbGetIteratorCount(mi);
        int erasing = 1;

        if (matches == 0) {
            rpmlog(RPMLOG_ERR, "包 %s 未安装。\n", pkg_name);
            numFailed++;
        } else {
            Header h; // 迭代器拥有引用

            if (matches > 1) {
                rpmlog(RPMLOG_ERR, "\"%s\" 匹配了多个包：\n", pkg_name);
                numFailed++;
                erasing = 0;
            }

            while ((h = rpmdbNextIterator(mi)) != NULL) {
                HeaderNode *newNode = (HeaderNode *)malloc(sizeof(HeaderNode));
                if (newNode == NULL) {
                    rpmlog(RPMLOG_ERR, "内存分配失败\n");
                    rpmdbFreeIterator(mi);
                    return;
                }
                newNode->header = h;
                newNode->next = NULL;

                // 将节点添加到链表
                if (tail == NULL) {
                    head = newNode;
                    tail = newNode;
                } else {
                    tail->next = newNode;
                    tail = newNode;
                }

                if (erasing) {
                    (void) rpmtsAddEraseElement(ts, h, -1);
                    numPackages++;
                } else {
                    char *nevra = headerFormat(h, qfmt, NULL);
                    rpmlog(RPMLOG_NOTICE, "  %s", nevra);
                    free(nevra);
                }
            }
        }
        // TODO: is mi need to free?
        //rpmdbFreeIterator(mi);
    }

    // 依赖检查
    rpmps ps;

    if (rpmtsCheck(ts)) {
        result = -1;
    }

    ps = rpmtsProblems(ts);
    if (result != -1 && rpmpsNumProblems(ps) > 0) {
        rpmlog(RPMLOG_ERR, "卸载依赖问题:\n");
        rpmpsPrint(NULL, ps);
        result = -1;
    }
    rpmpsFree(ps);

    // 执行卸载事务
    if (result != -1) {

        if (rpmtsOrder(ts)) {
            result = -1;
            goto cleanup;
        }

        // 执行 `%pre` 脚本
        for (arg = pkg_names; *arg; arg++) {
            const char *pkg_name = *arg;
            rpmdbMatchIterator mi = rpmtsInitIterator(ts, RPMDBI_LABEL, *arg, 0);
            int matches = rpmdbGetIteratorCount(mi);

            if (matches == 0) {
                rpmlog(RPMLOG_ERR, "Something wrong happened when removing %s. \n", pkg_name);
            } else {
                Header h; // 迭代器拥有引用

                if (matches > 1) {
                    rpmlog(RPMLOG_ERR, "\"%s\" 匹配了多个包：\n", pkg_name);
                }

                while ((h = rpmdbNextIterator(mi)) != NULL) {
                    if (execute_scripts_in_root(ts, NULL, h, RPMSCRIPT_PREUN) != RPMRC_OK) {
                        result = -1;
                        goto cleanup;
                    }
                }
            }
        }

        // 用于接收文件计数
        int file_count = 0;
        filelist = record_hardlinks_to_remove(ts, pkg_names, &file_count);

        // 执行卸载事务
        rpmFlags flags = 0;
        // 根据需要设置标志，例如忽略依赖等
        rpmtsSetFlags (ts, RPMTRANS_FLAG_NOSCRIPTS);
        rc = rpmtsRun(ts, NULL, flags);
        if (rc != RPMRC_OK && rc != RPMRC_NOTFOUND) {
            rpmlog(RPMLOG_ERR, "卸载失败:\n");

            // 获取事务的问题集合
            rpmps problems = rpmtsProblems(ts);

            if (problems != NULL) {
                if (rpmpsNumProblems(problems) > 0) {
                    rpmpsPrint(NULL, problems);
                }
                rpmpsFree(problems);
            } else {
                rpmlog(RPMLOG_ERR, "未能获取详细的错误信息。\n");
            }

            result = -1;
            goto cleanup;
        } else {
            rpmlog(RPMLOG_INFO, "卸载成功\n");
        }

        // remount the root, then create the hardlinks
        char *temp_dir = NULL;
        if (remount_root_rw(root_path, &temp_dir) != 0) {
            rpmlog(RPMLOG_ERR, "Failed to remount the root。\n");
            result = -1;
            goto cleanup;
        }

        // if temp_dir is NULL, we use the root directly
        if (temp_dir == NULL) {
            temp_dir = "/";
        }

        // 删除根目录的硬链接
        if (remove_hard_links_from_root(filelist, file_count, temp_dir) != 0) {
            result = -1;
            goto cleanup;
        }

        HeaderNode *current = head;
        while (current != NULL) {
            // 处理 current->header
            char *nevra = headerFormat(current->header, qfmt, NULL);
            rpmlog(RPMLOG_DEBUG, "Running scripts of %s ..\n", nevra);
            free(nevra);

            // 执行 `%post` 脚本
            if (execute_scripts_in_root(ts, NULL, current->header, RPMSCRIPT_POSTUN) != 0) {
                result = -1;
                goto cleanup;
            }

            // 执行 `%posttrans` 脚本
            if (execute_scripts_in_root(ts, NULL, current->header, RPMSCRIPT_POSTTRANS) != 0) {
                result = -1;
                goto cleanup;
            }

            current = current->next;
        }

        // umount the root, then remove the temporary directory
        if (temp_dir != NULL && temp_dir != "/") {
            if(umount_and_remove(temp_dir)){
                result = -1;
                goto cleanup;
            }
        }
    }

    if (numFailed) {
        result = -1;
        goto cleanup;
    }

cleanup:
    free(qfmt);
    if (ts != NULL) {
        rpmtsFree(ts);
    }
    return result;
}

// 查询 RPM 包信息的函数
int query_packages(const char **pkg_names, int pkg_count, const char *root_path) {
    rpmts ts = NULL;
    int result = 0;
    char * const * arg;

    // 初始化 RPM 库
    if (rpmReadConfigFiles(NULL, NULL) != 0) {
        rpmlog(RPMLOG_ERR, "无法读取 RPM 配置文件。\n");
        return -1;
    }

    // 创建事务集
    ts = rpmtsCreate();
    if (ts == NULL) {
        rpmlog(RPMLOG_ERR, "无法创建事务集。\n");
        return -1;
    }

    // 设置根路径
    if (rpmtsSetRootDir(ts, root_path) != 0) {
        rpmlog(RPMLOG_ERR, "无法设置根路径为 %s。\n", root_path);
        result = -1;
        goto cleanup;
    }

    // 遍历包名数组
    for (arg = pkg_names; *arg; arg++) {
        const char *pkg_name = *arg;
        rpmdbMatchIterator mi = rpmtsInitIterator(ts, RPMDBI_LABEL, *arg, 0);

        if (mi == NULL) {
            rpmlog(RPMLOG_WARNING, "包 %s 未安装。\n", pkg_name);
            rpmlog(RPMLOG_DEBUG, "无法创建数据库迭代器。\n");
            rpmdbFreeIterator(mi);
            continue;
            //result = -1;
            //goto cleanup;
        }

        int matches = rpmdbGetIteratorCount(mi);
        if (matches == 0) {
            rpmlog(RPMLOG_WARNING, "包 %s 未安装。\n", pkg_name);
            rpmdbFreeIterator(mi);
            continue;
        }

        Header h;
        while ((h = rpmdbNextIterator(mi)) != NULL) {
            // 获取包的属性
            char *name = headerGetString(h, RPMTAG_NAME);
            char *version = headerGetString(h, RPMTAG_VERSION);
            char *release = headerGetString(h, RPMTAG_RELEASE);
            char *arch = headerGetString(h, RPMTAG_ARCH);
            char *summary = headerGetString(h, RPMTAG_SUMMARY);
            uint32_t install_time_uint32 = headerGetNumber(h, RPMTAG_INSTALLTIME);
            time_t install_time = (time_t)install_time_uint32;
            rpmlog(RPMLOG_DEBUG, "时间戳：%d\n", install_time);

            // 如果 install_time 为 0，可能标签不存在或获取失败
            if (install_time == 0) {
                rpmlog(RPMLOG_WARNING, "无法获取包 %s 的安装时间。\n", name ? name : "未知");
            }

            // 格式化安装时间
            char time_buf[64] = {0};
            struct tm *timeinfo = localtime(&install_time);
            if (timeinfo != NULL) {
                strftime(time_buf, sizeof(time_buf), "%Y-%m-%d %H:%M:%S", timeinfo);
            } else {
                strcpy(time_buf, "未知时间");
            }

            // 输出包信息
            rpmlog(RPMLOG_INFO, "包名       ：%s\n", name ? name : "N/A");
            rpmlog(RPMLOG_INFO, "版本       ：%s\n", version ? version : "N/A");
            rpmlog(RPMLOG_INFO, "发行版     ：%s\n", release ? release : "N/A");
            rpmlog(RPMLOG_INFO, "架构       ：%s\n", arch ? arch : "N/A");
            rpmlog(RPMLOG_INFO, "安装时间   ：%s\n", time_buf);
            rpmlog(RPMLOG_INFO, "摘要       ：%s\n", summary ? summary : "N/A");
            rpmlog(RPMLOG_INFO, "---------------------------------------\n");
        }

        rpmdbFreeIterator(mi);
    }

cleanup:
    if (ts != NULL) {
        rpmtsFree(ts);
    }
    return result;
}