// utils.c

#include <stdio.h>
#include <sys/stat.h>
#include <libgen.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <rpm/rpmlog.h>
#include <sys/mount.h>
#include "utils.h"

int copy_file(const char *source, const char *dest) {
    FILE *src = fopen(source, "rb");
    if (src == NULL) {
        fprintf(stderr, "无法打开源文件 %s: %s\n", source, strerror(errno));
        return -1;
    }

    FILE *dst = fopen(dest, "wb");
    if (dst == NULL) {
        fprintf(stderr, "无法创建目标文件 %s: %s\n", dest, strerror(errno));
        fclose(src);
        return -1;
    }

    char buf[8192];
    size_t n;
    while ((n = fread(buf, 1, sizeof(buf), src)) > 0) {
        if (fwrite(buf, 1, n, dst) != n) {
            fprintf(stderr, "写入目标文件失败: %s\n", strerror(errno));
            fclose(src);
            fclose(dst);
            return -1;
        }
    }

    fclose(src);
    fclose(dst);

    return 0;
}

int create_parent_dirs(const char *path) {
    char *path_copy = strdup(path);
    if (path_copy == NULL) {
        fprintf(stderr, "内存分配失败。\n");
        return -1;
    }

    char *dir = dirname(path_copy);
    if (mkdir_p(dir) != 0) {
        fprintf(stderr, "创建目录 %s 失败。\n", dir);
        free(path_copy);
        return -1;
    }

    free(path_copy);
    return 0;
}

int is_mount_point(const char *path) {
    FILE *file = fopen("/proc/mounts", "r");
    if (file == NULL) {
        return -1;
    }

    char real_path[PATH_MAX];
    if (realpath(path, real_path) == NULL) {
        fclose(file);
        return -1;
    }

    char line[256];
    while (fgets(line, sizeof(line), file) != NULL) {
        // 使用 strtok 分割字符串
        char *token = strtok(line, " ");
        if (token == NULL) {
            continue;
        }

        // 获取第二个字段，即挂载路径
        char *mounted_path = strtok(NULL, " ");
        if (mounted_path == NULL) {
            continue;
        }

        char real_mounted_path[PATH_MAX];
        if (realpath(mounted_path, real_mounted_path) == NULL) {
            continue;
        }

        rpmlog(RPMLOG_DEBUG, "mounted_path: %s\n", real_mounted_path);

        if (strcmp(real_mounted_path, real_path) == 0) {
            rpmlog(RPMLOG_DEBUG, "Path %s is a mount point.\n", path);
            fclose(file);
            return 1;
        }
    }

    fclose(file);
    return 0;
}

int umount_and_remove(const char *path) {
    int result = 0;
    int retry_count = 0;
    const int max_retries = 5;  // 设置最大重试次数

    while (retry_count < max_retries) {
        errno = 0;

        if (is_mount_point(path)) {
            if (umount2(path, MNT_FORCE) != 0) {
                rpmlog(RPMLOG_ERR, "Failed to unmount the temporary directory %s: %s \n", path, strerror(errno));
                result = -1;
            } else {
                rpmlog(RPMLOG_DEBUG, "Successfully unmounted the temporary directory: %s\n", path);
            }
            retry_count++;
        } else {
            break;
        }
    }

    if (rmdir(path) != 0) {
        rpmlog(RPMLOG_ERR, "Failed to remove the temporary directory %s:%s \n", path, strerror(errno));
        result = -1;
    }

    return result;
}

int mkdir_p(const char *dir) {
    char tmp[PATH_MAX];
    char *p = NULL;
    size_t len;

    snprintf(tmp, sizeof(tmp), "%s", dir);
    len = strlen(tmp);
    if (tmp[len - 1] == '/')
        tmp[len - 1] = '\0';

    for (p = tmp + 1; *p; p++)
        if (*p == '/') {
            *p = '\0';
            if (mkdir(tmp, S_IRWXU) != 0) {
                if (errno != EEXIST) {
                    return -1;
                }
            }
            *p = '/';
        }

    if (mkdir(tmp, S_IRWXU) != 0) {
        if (errno != EEXIST) {
            return -1;
        }
    }

    return 0;
}